@@ -146,6 +146,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Oceanic", "Veo 180",   DC_FAMILY_OCEANIC_VEO250, 0x4252},
 	{"Aeris",   "XR-2",      DC_FAMILY_OCEANIC_VEO250, 0x4255},
 	{"Sherwood", "Insight",  DC_FAMILY_OCEANIC_VEO250, 0x425A},
+	{"Hollis",  "DG02",      DC_FAMILY_OCEANIC_VEO250, 0x4352},
 	/* Oceanic Atom 2.0 */
 	{"Oceanic",  "Atom 1.0",            DC_FAMILY_OCEANIC_ATOM2, 0x4250},
 	{"Aeris",    "Epic",                DC_FAMILY_OCEANIC_ATOM2, 0x4257},
@@ -194,13 +195,15 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Oceanic",  "OCi",                 DC_FAMILY_OCEANIC_ATOM2, 0x454B},
 	{"Aeris",    "A300CS",              DC_FAMILY_OCEANIC_ATOM2, 0x454C},
 	{"Beuchat",  "Mundial 3",           DC_FAMILY_OCEANIC_ATOM2, 0x4550},
+	{"Oceanic",  "F10",                 DC_FAMILY_OCEANIC_ATOM2, 0x4553},
 	{"Oceanic",  "F11",                 DC_FAMILY_OCEANIC_ATOM2, 0x4554},
 	{"Subgear",  "XP-Air",              DC_FAMILY_OCEANIC_ATOM2, 0x4555},
 	{"Sherwood", "Vision",              DC_FAMILY_OCEANIC_ATOM2, 0x4556},
 	{"Oceanic",  "VTX",                 DC_FAMILY_OCEANIC_ATOM2, 0x4557},
 	{"Aqualung", "i300",                DC_FAMILY_OCEANIC_ATOM2, 0x4559},
+	{"Aqualung", "i750TC",              DC_FAMILY_OCEANIC_ATOM2, 0x455A},
 	{"Aqualung", "i450T",               DC_FAMILY_OCEANIC_ATOM2, 0x4641},
-	{"Aqualung", "i550T",               DC_FAMILY_OCEANIC_ATOM2, 0x4642},
+	{"Aqualung", "i550",                DC_FAMILY_OCEANIC_ATOM2, 0x4642},
 	/* Mares Nemo */
 	{"Mares", "Nemo",         DC_FAMILY_MARES_NEMO, 0},
 	{"Mares", "Nemo Steel",   DC_FAMILY_MARES_NEMO, 0},
@@ -251,6 +254,7 @@ static const dc_descriptor_t g_descriptors[] = {
 	{"Cressi", "Leonardo", DC_FAMILY_CRESSI_LEONARDO, 1},
 	{"Cressi", "Giotto",   DC_FAMILY_CRESSI_LEONARDO, 4},
 	{"Cressi", "Newton",   DC_FAMILY_CRESSI_LEONARDO, 5},
+	{"Cressi", "Drake",    DC_FAMILY_CRESSI_LEONARDO, 6},
 	/* Zeagle N2iTiON3 */
 	{"Zeagle",   "N2iTiON3",   DC_FAMILY_ZEAGLE_N2ITION3, 0},
 	{"Apeks",    "Quantum X",  DC_FAMILY_ZEAGLE_N2ITION3, 0},
