@@ -2,7 +2,6 @@
  * libdivecomputer
  *
  * Copyright (C) 2008 Jef Driesen
- * Copyright (C) 2015 Claudiu Olteanu
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -130,7 +129,7 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 		rc = reefnet_sensusultra_device_open (&device, context, name);
 		break;
 	case DC_FAMILY_OCEANIC_VTPRO:
-		rc = oceanic_vtpro_device_open (&device, context, name);
+		rc = oceanic_vtpro_device_open2 (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
 	case DC_FAMILY_OCEANIC_VEO250:
 		rc = oceanic_veo250_device_open (&device, context, name);
@@ -198,33 +197,6 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 	return rc;
 }
 
-dc_status_t
-dc_device_custom_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descriptor, dc_serial_t *serial)
-{
-	dc_status_t rc = DC_STATUS_SUCCESS;
-	dc_device_t *device = NULL;
-
-	if (out == NULL || descriptor == NULL || serial == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	switch (dc_descriptor_get_type (descriptor)) {
-	case DC_FAMILY_HW_OSTC3:
-		rc = hw_ostc3_device_custom_open (&device, context, serial);
-		break;
-	case DC_FAMILY_SHEARWATER_PREDATOR:
-		rc = shearwater_predator_device_custom_open (&device, context, serial);
-		break;
-	case DC_FAMILY_SHEARWATER_PETREL:
-		rc = shearwater_petrel_device_custom_open (&device, context, serial);
-		break;
-	default:
-		return DC_STATUS_INVALIDARGS;
-	}
-
-	*out = device;
-
-	return rc;
-}
 
 int
 dc_device_isinstance (dc_device_t *device, const dc_device_vtable_t *vtable)
