@@ -319,6 +319,8 @@ suunto_vyper_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
+	unsigned int gauge = data[4] & 0x40;
+
 	// Time
 	sample.time = 0;
 	if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
@@ -327,6 +329,12 @@ suunto_vyper_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 	sample.depth = 0;
 	if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 
+	// Initial gas mix
+	if (!gauge) {
+		sample.gasmix = 0;
+		if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+	}
+
 	unsigned int depth = 0;
 	unsigned int time = 0;
 	unsigned int interval = data[3];
@@ -394,12 +402,7 @@ suunto_vyper_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t
 
 				sample.gasmix = idx;
 				if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
-#ifdef ENABLE_DEPRECATED
-				sample.event.type = SAMPLE_EVENT_GASCHANGE;
-				sample.event.value = o2;
-#else
 				sample.event.type = SAMPLE_EVENT_NONE;
-#endif
 				break;
 			default: // Unknown
 				WARNING (abstract->context, "Unknown event");
