@@ -216,6 +216,9 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 
 	unsigned int time = 20;
 
+	unsigned int gasmix_previous = 0xFFFFFFFF;
+	unsigned int gasmix = 0;
+
 	unsigned int offset = header + 18;
 	while (offset + 2 <= size) {
 		dc_sample_value_t sample = {0};
@@ -233,6 +236,13 @@ uwatec_memomouse_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callba
 		sample.depth = depth * 10.0 / 64.0;
 		if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 
+		// Gas change.
+		if (gasmix != gasmix_previous) {
+			sample.gasmix = gasmix;
+			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
+			gasmix_previous = gasmix;
+		}
+
 		// Warnings
 		for (unsigned int i = 0; i < 6; ++i) {
 			if (warnings & (1 << i)) {
